@@ -1,3 +1,4 @@
+# -*- test-case-name: formless.test -*-
 # Copyright (c) 2004 Divmod.
 # See LICENSE for details.
 
@@ -10,6 +11,7 @@ import sys
 import copy
 import inspect
 import types
+import warnings
 
 from nevow import inevow
 from nevow import util
@@ -41,8 +43,20 @@ class InputError(Exception):
 
 
 class ValidateError(Exception):
-    """A Binding instance was unable to coerce all it's arguments
-    from a dictionary of lists of strings to the appropriate types.
+    """A Binding instance was unable to coerce all it's arguments from a
+    dictionary of lists of strings to the appropriate types.
+
+    One use of this is to raise from an autocallable if an input is invalid.
+    For example, a password is incorrect.
+    
+    errors must be a dictionary mapping argument names to error messages
+    to display next to the arguments on the form.
+
+    formErrorMessage is a string to display at the top of the form, not tied to
+    any specific argument on the form.
+
+    partialForm is a dict mapping argument name to argument value, allowing
+    you to have the form save values that were already entered in the form.
     """
     def __init__(self, errors, formErrorMessage=None, partialForm=None):
         self.errors = errors
@@ -62,34 +76,62 @@ class Typed(object):
     TypedInterface class definitions to describe the types of properties,
     the parameter types to method calls, and method return types.
     
-    label: The short label which will describe this parameter/proerties
-        purpose to the user.
+    @ivar label: The short label which will describe this
+        parameter/proerties purpose to the user.
     
-    description: A long description which further describes the sort
-        of input the user is expected to provide.
+    @ivar description: A long description which further describes the
+        sort of input the user is expected to provide.
     
-    default: A default value that may be used as an initial value in
-        the form.
+    @ivar default: A default value that may be used as an initial
+        value in the form.
 
-    required: Whether the user is required to provide a value
+    @ivar required: Whether the user is required to provide a value
 
-    null: The value which will be produced if required is False and
-        the user does not provide a value
+    @ivar null: The value which will be produced if required is False
+        and the user does not provide a value
+
+    @ivar unicode: Iff true, try to determine the character encoding
+        of the data from the browser and pass unicode strings to
+        coerce.
     """
     __implements__ = iformless.ITyped
 
     complexType = False
     strip = False
+    label = None
+    description = None
+    default = ''
+    required = False
     requiredFailMessage = 'Please enter a value'
-    def __init__(self, label='', description='', default='', required=False, requiredFailMessage=None, null=None, **attributes):
+    null = None
+    unicode = False
+
+    def __init__(
+        self,
+        label=None,
+        description=None,
+        default=None,
+        required=None,
+        requiredFailMessage=None,
+        null=None,
+        unicode=None,
+        **attributes):
+
         self.id = nextId()
-        self.label = label
-        self.description = description
-        self.default = default
-        self.required = required
+        if label is not None:
+            self.label = label
+        if description is not None:
+            self.description = description
+        if default is not None:
+            self.default = default
+        if required is not None:
+            self.required = required
         if requiredFailMessage is not None:
             self.requiredFailMessage = requiredFailMessage
-        self.null = null
+        if null is not None:
+            self.null = null
+        if unicode is not None:
+            self.unicode = unicode
         self.attributes = attributes
 
     def getAttribute(self, name, default=None):
@@ -112,13 +154,15 @@ class String(Typed):
     """
 
     requiredFailMessage = 'Please enter a string.'
+    # iff true, return the stripped value.
+    strip = False
 
     def __init__(self, *args, **kwargs):
-        if 'strip' in kwargs:
+        try:
             self.strip = kwargs['strip']
             del kwargs['strip']
-        else:
-            self.strip = False
+        except KeyError:
+            pass
         Typed.__init__(self, *args, **kwargs)
 
     def coerce(self, val, configurable):
@@ -168,9 +212,9 @@ class Integer(Typed):
                 try:
                     return long(val)
                 except ValueError:
-                    raise InputError("%r is not an integer." % val)
+                    raise InputError("'%s' is not an integer." % val)
             
-            raise InputError("%r is not an integer." % val)
+            raise InputError("'%s' is not an integer." % val)
 
 
 class Real(Typed):
@@ -185,7 +229,7 @@ class Real(Typed):
         try:
             return float(val)
         except ValueError:
-            raise InputError("%r is not a real number." % val)
+            raise InputError("'%s' is not a real number." % val)
 
 
 class Boolean(Typed):
@@ -194,7 +238,7 @@ class Boolean(Typed):
             return False
         elif val == 'True':
             return True
-        raise InputError("%r is not a boolean" % val)
+        raise InputError("'%s' is not a boolean" % val)
 
 
 class FixedDigitInteger(Integer):
@@ -222,38 +266,77 @@ class Directory(Typed):
         if val is None:
             return None
         if not os.path.exists(val):
-            raise InputError("The directory %r does not exist." % val)
+            raise InputError("The directory '%s' does not exist." % val)
         return val
 
 
 class Choice(Typed):
-    """Allow the user to pick from a list of "choices", or a list of choices found
-    by accessing the list in the attribute "choicesAttribute" of the object we
-    are configuring. The elements of the list will be rendered by calling the 
-    function passed to stringify, which is by default "str".
+    """Allow the user to pick from a list of "choices", presented in a drop-down
+    menu. The elements of the list will be rendered by calling the function
+    passed to stringify, which is by default "str".
     """
-    def __init__(self, choices=None, choicesAttribute=None, stringify=str, *args, **kw):
-        Typed.__init__(self, *args, **kw)
-        if choices is None:
-            self.choices = []
-        else:
-            self.choices = choices
-        self.choicesAttribute = choicesAttribute
-        self.stringify = stringify
 
-    def coerce(self, val, binding):
+    def __init__(self, choices=None, choicesAttribute=None, stringify=str,
+                 valueToKey=str, keyToValue=None, keyAndConfigurableToValue=None,
+                 *args, **kw):
+        """
+        Create a Choice.
+
+        @param choices: an object adaptable to IGettable for an iterator (such
+        as a function which takes (ctx, data) and returns a list, a list
+        itself, a tuple, a generator...)
+
+        @param stringify: a pretty-printer.  a function which takes an object
+        in the list of choices and returns a label for it.
+
+        @param valueToKey: a function which converts an object in the list of
+        choices to a string that can be sent to a client.
+
+        @param keyToValue: a 1-argument convenience version of
+        keyAndConfigurableToValue
+
+        @param keyAndConfigurableToValue:  a 2-argument function which takes a string such as
+        one returned from valueToKey and a configurable, and returns an object
+        such as one from the list of choices.
+        """
+
+        Typed.__init__(self, *args, **kw)
+        self.choices = choices
+        if choicesAttribute:
+            self.choicesAttribute = choicesAttribute
+        if getattr(self, 'choicesAttribute', None):
+            warnings.warn(
+                "Choice.choicesAttribute is deprecated. Please pass a function to choices instead.",
+                DeprecationWarning,
+                stacklevel=2)
+            def findTheChoices(ctx, data):
+                return getattr(iformless.IConfigurable(ctx).original, self.choicesAttribute)
+            self.choices = findTheChoices
+
+        self.stringify = stringify
+        self.valueToKey=valueToKey
+
+        if keyAndConfigurableToValue is not None:
+            assert keyToValue is None, 'This should be *obvious*'
+            self.keyAndConfigurableToValue = keyAndConfigurableToValue
+        elif keyToValue is not None:
+            self.keyAndConfigurableToValue = lambda x,y: keyToValue(x)
+        else:
+            self.keyAndConfigurableToValue = lambda x,y: str(x)
+
+
+    def coerce(self, val, configurable):
         """Coerce a value with the help of an object, which is the object
         we are configuring.
         """
-        try:
-            val = int(val)
-        except ValueError:
-            raise InputError("%r is an invalid choice." % val)
-        if self.choicesAttribute is not None:
-            choices = getattr(binding, self.choicesAttribute)
-        else:
-            choices = self.choices
-        return choices[val]
+        return self.keyAndConfigurableToValue(val, configurable)
+
+
+class Radio(Choice):
+    """Type influencing presentation! horray!
+
+    Show the user radio button choices instead of a picklist.
+    """
 
 
 class Any(object):
@@ -333,9 +416,9 @@ class Request(Typed):
     request when called. Including a Request arg will not affect the
     appearance of the rendered form.
 
-    def doSomething(self, request=formless.Request(), name=formless.String()):
-        pass
-    doSomething = formless.autocallable(doSomething)
+    >>> def doSomething(self, request=formless.Request(), name=formless.String()):
+    ...     pass
+    >>> doSomething = formless.autocallable(doSomething)
     """
     complexType = True ## Don't use the regular form
 
@@ -345,9 +428,9 @@ class Context(Typed):
     context when called. Including a Context arg will not affect the
     appearance of the rendered form.
 
-    def doSomething(self, context=formless.Context(), name=formless.String()):
-        pass
-    doSomething = formless.autocallable(doSomething)
+    >>> def doSomething(self, context=formless.Context(), name=formless.String()):
+    ...     pass
+    >>> doSomething = formless.autocallable(doSomething)
     """
     complexType = True ## Don't use the regular form
 
@@ -389,13 +472,13 @@ def autocallable(method, action=None, visible=False, **kw):
     
     Use this like a method adapter around a method in a TypedInterface:
     
-    class IFoo(TypedInterface):
-        def doSomething(self):
-            '''Do Something
-            
-            Do some action bla bla'''
-            return None
-        doSomething = autocallable(doSomething, action="Do it!!")
+    >>> class IFoo(TypedInterface):
+    ...     def doSomething(self):
+    ...         '''Do Something
+    ...         
+    ...         Do some action bla bla'''
+    ...         return None
+    ...     doSomething = autocallable(doSomething, action="Do it!!")
     """
     method.autocallable = True
     method.id = nextId()
@@ -431,16 +514,20 @@ class Binding(object):
     """
     __implements__ = iformless.IBinding,
 
+    label = None
+    description = ''
+
     def __init__(self, name, typedValue, id=0):
         self.id = id
         self.name = name
         self.typedValue = iformless.ITyped(typedValue)
 
         # pull these out to remove one level of indirection...
-        self.description = typedValue.description
-        if typedValue.label:
+        if typedValue.description is not None:
+            self.description = typedValue.description
+        if typedValue.label is not None:
             self.label = typedValue.label
-        else:
+        if self.label is None:
             self.label = nameToLabel(name)
         self.default = typedValue.default
         self.complexType = typedValue.complexType
@@ -451,7 +538,7 @@ class Binding(object):
     def getArgs(self):
         """Return a *copy* of this Binding.
         """
-        return (Binding(self.name, self.original, self.id), )
+        Return (Binding(self.name, self.original, self.id), )
 
     def getViewName(self):
         return self.original.__class__.__name__.lower()
@@ -589,14 +676,14 @@ def nameToLabel(mname):
     return ' '.join(labelList)
 
 
-def labelAndDescriptionFromNameAndDocstring(name, docstring):
+def labelAndDescriptionFromDocstring(docstring):
     if docstring is None:
         docstring = ''
     docs = filter(lambda x: x, [x.strip() for x in docstring.split('\n')])
     if len(docs) > 1:
         return docs[0], '\n'.join(docs[1:])
     else:
-        return nameToLabel(name), '\n'.join(docs)
+        return None, '\n'.join(docs)
 
 
 class MetaTypedInterface(MetaInterface):
@@ -605,30 +692,32 @@ class MetaTypedInterface(MetaInterface):
     described in the Binding docstring occurs, and when it is all done, there will
     be three attributes on the TypedInterface class:
     
-    __methods__: An ordered list of all the MethodBinding instances produced
-        by introspecting all autocallable methods on this TypedInterface
+     - __methods__: An ordered list of all the MethodBinding instances
+       produced by introspecting all autocallable methods on this
+       TypedInterface
 
-    __properties__: An ordered list of all the Property Binding instances produced
-        by introspecting all properties which have Typed values on this TypedInterface
+     - __properties__: An ordered list of all the Property Binding
+       instances produced by introspecting all properties which have
+       Typed values on this TypedInterface
 
-    __spec__: An ordered list of all methods and properties
+     - __spec__: An ordered list of all methods and properties
     
     These lists are sorted in the order that the methods and properties appear
     in the TypedInterface definition.
     
     For example:
     
-    class Foo(TypedInterface):
-        bar = String()
-        baz = Integer()
-        
-        def frotz(self): pass
-        frotz = autocallable(frotz)
-        
-        xyzzy = Float()
-        
-        def blam(self): pass
-        blam = autocallable(blam)
+    >>> class Foo(TypedInterface):
+    ...     bar = String()
+    ...     baz = Integer()
+    ...     
+    ...     def frotz(self): pass
+    ...     frotz = autocallable(frotz)
+    ...     
+    ...     xyzzy = Float()
+    ...     
+    ...     def blam(self): pass
+    ...     blam = autocallable(blam)
 
     Once the metaclass __new__ is done, the Foo class instance will have three
     properties, __methods__, __properties__, and __spec__,
@@ -665,11 +754,40 @@ class MetaTypedInterface(MetaInterface):
                     argumentTypes = [
                         Argument(n, argtype, argtype.id) for n, argtype in zip(names[1:], typeList)
                     ]
-                label, description = labelAndDescriptionFromNameAndDocstring(key, value.__doc__)
+                label = None
+                description = None
+                if getattr(value, 'autocallable', None):
+                    # autocallables have attributes that can set label and description
+                    label = value.attributes.get('label', None)
+                    description = value.attributes.get('description', None)
 
                 adapted = iformless.ITyped(result, None)
                 if adapted is None:
                     adapted = Object(result)
+
+                # ITyped has label and description we can use
+                if label is None:
+                    label = adapted.label
+                if description is None:
+                    description = adapted.description
+
+                defaultLabel, defaultDescription = labelAndDescriptionFromDocstring(value.__doc__)
+                if defaultLabel is None:
+                    # docstring had no label, try the action if it is an autocallable
+                    if getattr(value, 'autocallable', None):
+                        if label is None and value.action is not None:
+                            # no explicit label, but autocallable has action we can use
+                            defaultLabel = value.action
+
+                if defaultLabel is None:
+                    # final fallback: use the function name as label
+                    defaultLabel = nameToLabel(key)
+                    
+                if label is None:
+                    label = defaultLabel
+                if description is None:
+                    description = defaultDescription
+
                 theMethod = Method(
                     adapted, argumentTypes, label=label, description=description
                 )
@@ -695,7 +813,18 @@ class MetaTypedInterface(MetaInterface):
         dct['__spec__'] = spec = methods + properties
         spec.sort(_sorter)
         dct['name'] = name
-        dct['label'], dct['description'] = labelAndDescriptionFromNameAndDocstring(name, dct.get('__doc__'))
+
+        # because attributes "label" and "description" would become Properties,
+        # check for ones with an underscore prefix.
+        dct['label'] = dct.get('_label', None)
+        dct['description'] = dct.get('_description', None)
+        defaultLabel, defaultDescription = labelAndDescriptionFromDocstring(dct.get('__doc__'))
+        if defaultLabel is None:
+            defaultLabel = nameToLabel(name)
+        if dct['label'] is None:
+            dct['label'] = defaultLabel
+        if dct['description'] is None:
+            dct['description'] = defaultDescription
 
         # What the heck...work around strange bug in Zope C Optimizations
         # whose cause I cannot figure out.
