@@ -14,7 +14,10 @@ try:
     import twisted
     sys.exit(os.system('trial -v nevow.test formless.test'))
 except ImportError:
-    skips.extend(['nevow.test.test_disktemplate', 'nevow.test.test_later'])
+    skips.extend([
+        'nevow.test.test_disktemplate',
+        'nevow.test.test_later',
+        'formless.test.test_freeform'])
 
 
 import nevow.test, formless.test
